import os
import re
import subprocess

from PIL import Image


class LXIDevice:
    @staticmethod
    def _system_command(cmd):
        return subprocess.check_output(cmd, shell=True, text=True)

    @staticmethod
    def _get_address(name):
        cmd = 'lxi discover'
        stdout = LXIDevice._system_command(cmd)
        found_str = re.search(f'Found.*{name}.*', stdout).group(0)
        return re.search('address.*', found_str).group(0)[len('address '):]

    @staticmethod
    def _get_name(address):
        cmd = 'lxi discover'
        stdout = LXIDevice._system_command(cmd)
        found_str = re.search(f'Found.*{address}.*', stdout).group(0)
        return re.search('\".*\"', found_str).group(0)[1:-1]

    def __init__(self, name=None, address=None):
        if address is not None:
            self.address = address
            try:
                self.name = self._get_name(address)
            except AttributeError:
                raise RuntimeError('Device not connected')
        elif name is not None:
            try:
                self.address = self._get_address(name)
                self.name = self._get_name(self.address)
            except AttributeError:
                raise RuntimeError('Device not connected')
        else:
            raise ValueError('Need either address or name of the device')

    def _is_connected(self):
        return self.address == self._get_address(self.address)

    def capture_screenshot(self, path='~/remote_screenshot.png'):
        path = os.path.expanduser(path)
        path_no_ext = os.path.splitext(path)[0]
        path_bmp = path_no_ext + '.bmp'

        cmd = f'lxi screenshot -a {self.address} {path_bmp}'
        self._system_command(cmd)

        Image.open(path_bmp).save(path)

        return path

    def send_command(self, command):
        cmd = f'lxi scpi -a {self.address} "{command}"'
        return self._system_command(cmd)



from pyngrok import conf, ngrok

_type_description = {
    'ssh': {
        'port': 22,
        'proto': 'tcp',
    },
    'custom': {
        'port': 1209,
        'proto': 'tcp',
    },
    'jupyter': {
        'port': 8888,
        'proto': 'http',
    },
    'grafana': {
        'port': 3000,
        'proto': 'http',
    },
    'camera': {
        'port': 56000,
        'proto': 'http',
    }
}
HOST = 'localhost'


def _get_addr_from_url(url):
    return url.split("//")[-1].split(":")[0]


def _get_port_from_url(url):
    return url.split("//")[-1].split(":")[1]


class TunnelAlredyOpenError(Exception):
    pass


class Tunnel:
    @staticmethod
    def _is_already_open(tunnel_type):
        PORT = str(_type_description[tunnel_type]['port'])

        if _type_description[tunnel_type]['proto'] == 'tcp':
            this_tunnel_config = f'{HOST}:{PORT}'
        elif _type_description[tunnel_type]['proto'] == 'http':
            this_tunnel_config = f'http://{HOST}:{PORT}'
        else:
            raise ValueError('Unexpected tunnel type')

        tunnels = ngrok.get_tunnels()
        if tunnels is None:
            return False
        for tunnel in tunnels:
            if tunnel.config['addr'] == this_tunnel_config:
                return True
        return False

    def __init__(self, tunnel_type):
        ngrok.get_tunnels()
        if Tunnel._is_already_open(tunnel_type):
            raise TunnelAlredyOpenError(
                "Such tunnel is already open"
            )
        self.ngrok_tunnel = ngrok.connect(
            _type_description[tunnel_type]['port'],
            _type_description[tunnel_type]['proto'],
        )
        self.address = _get_addr_from_url(self.ngrok_tunnel.public_url)
        if tunnel_type == 'ssh':
            self.port = _get_port_from_url(self.ngrok_tunnel.public_url)

    def is_still_open(self):
        tunnels = ngrok.get_tunnels()
        for tunnel in tunnels:
            if tunnel.public_url == self.ngrok_tunnel.public_url:
                return True
        return False

    def close(self):
        ngrok.disconnect(self.ngrok_tunnel.public_url)
        if self.ngrok_tunnel.proto == 'http':
            url = _get_addr_from_url(self.ngrok_tunnel.public_url)
            https_pub_url = 'https://' + url
            ngrok.disconnect(https_pub_url)


def authenticate(token):
    ngrok.set_auth_token(token)


def set_region(region):
    conf.get_default().region = region



import argparse
import configparser
import os

import questionary


PROJECT_NAME = 'remote_mole'

AVAILABLE_BOTS = [
    'Discord',
]

CONFIG_PATH = os.path.join(
    os.path.expanduser('~'),
    f'.config/{PROJECT_NAME}/config.toml',
)

SYSD_SERVICE_PATH = os.path.join(
    os.path.expanduser('~'),
    f'.config/systemd/user/{PROJECT_NAME}.service'
)

PY_EXEC_PATH = '/usr/bin/python3'


def _create_file_with_content(filename, config):
    dirname = os.path.dirname(filename)
    os.mkdir
    try:
        os.makedirs(dirname)
    except FileExistsError:
        pass

    with open(filename, "w") as f:
        config.write(f)


def register():
    platform_config = configparser.ConfigParser()
    try:
        bot_token = questionary.text('Discord bot token').unsafe_ask()
        bot_keyword = questionary.text(
            'To what keyword should the bot listen',
        ).unsafe_ask()
        platform_config['Discord'] = {
            'token': bot_token,
            'keyword': bot_keyword,
        }

        auth_ngrok = questionary.confirm(
            'Do you want to set up a ngrok token (recommended)',
            auto_enter=False,
        ).unsafe_ask()

        ngrok_data = {}
        if auth_ngrok:
            ngrok_token = questionary.text('ngrok token').unsafe_ask()
            ngrok_data['ngrok_token'] = ngrok_token

        ngrok_region = questionary.select(
            'ngrok region',
            choices=[
                {'name': 'South America', 'value': 'sa'},
                {'name': 'United States', 'value': 'us'},
                {'name': 'Europe', 'value': 'eu'},
                {'name': 'Asia or Pacific', 'value': 'ap'},
                {'name': 'Australia', 'value': 'au'},
                {'name': 'Japan', 'value': 'jp'},
                {'name': 'India', 'value': 'in'},
            ]
        ).unsafe_ask()

    except KeyboardInterrupt:
        print("Cancelled by user")
        return

    ngrok_data['region'] = ngrok_region
    platform_config['ngrok'] = ngrok_data

    _create_file_with_content(
        CONFIG_PATH,
        platform_config,
    )


def start():
    config = configparser.ConfigParser()
    config.optionxform = str
    config.read(CONFIG_PATH)

    sysd_file = configparser.ConfigParser()
    sysd_file.optionxform = str

    this_file = os.path.realpath(__file__)
    pkg_dir = os.path.dirname(os.path.dirname(this_file))

    if 'Discord' in config.sections():
        daemon_path = os.path.join(
            pkg_dir,
            '_internal/platforms/discord/daemon.py'
        )
    else:
        raise EnvironmentError('No platforms are configured!')

    sysd_file['Unit'] = {
        'Description': PROJECT_NAME,
    }
    sysd_file['Service'] = {
        'ExecStart': f'{PY_EXEC_PATH} {daemon_path}',
        'Restart': 'on-failure',
        'RestartSec': '60',
        'Environment': 'PYTHONUNBUFFERED=1',
    }
    sysd_file['Install'] = {
        'WantedBy': 'default.target',
    }

    _create_file_with_content(
        SYSD_SERVICE_PATH,
        sysd_file,
    )
    os.system('systemctl --user daemon-reload')
    os.system(f'systemctl --user enable {PROJECT_NAME}')

    os.system(f'systemctl --user start {PROJECT_NAME}')


def stop():
    os.system(f'systemctl --user stop {PROJECT_NAME}')


def status():
    os.system(f'systemctl --user status {PROJECT_NAME}')


def main():
    parser = argparse.ArgumentParser(
        description=PROJECT_NAME
    )
    parser.add_argument(
        '-register', action='store_true',
        help='Register or overwrite mole configuration',
    )
    parser.add_argument(
        '-start', action='store_true',
        help='Start the mole daemon',
    )
    parser.add_argument(
        '-stop', action='store_true',
        help='Stop the mole daemon',
    )
    parser.add_argument(
        '-status', action='store_true',
        help='Get service status from systemctl',
    )
    args = parser.parse_args()

    if args.register:
        register()
        return 0

    if args.start:
        start()
        return 0

    if args.stop:
        stop()
        return 0

    if args.status:
        status()
        return 0


if __name__ == "__main__":
    main()
